"""
PVSFolderWorkflow.py (inspiration: ForumWorkflow.py de CMFBoard)
"""

from Products.CMFCore.CMFCorePermissions import \
     View, ModifyPortalContent, AccessContentsInformation, \
     ListFolderContents

from Products.CMFCore.WorkflowTool import addWorkflowFactory
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
from Products.DCWorkflow.Transitions import TRIGGER_AUTOMATIC
from Products.PythonScripts.PythonScript import manage_addPythonScript

import os

from PVSPermissions import CreateVersions

from PVSPermissions import *

p_access = AccessContentsInformation
p_modify = ModifyPortalContent
p_view = View
p_list = ListFolderContents

r_anon = 'Anonymous'
r_manager = 'Manager'
r_owner = 'Owner'
r_auth = 'Authenticated'


def setupPVSFolderWorkflow(wf):
    wf.setProperties(title='PVS Folder Workflow')

    for s in ('private','member','published'):
        wf.states.addState(s)
    for t in ('publish','hide','show_members'):
        wf.transitions.addTransition(t)
    for v in ('action', 'actor', 'comments', 'review_history', 'time'):
        wf.variables.addVariable(v)
    for p in (p_access, p_view, p_list, p_modify,):
        wf.addManagedPermission(p)

    # states
    wf.states.setInitialState('published')

    sdef = wf.states['private']
    sdef.setProperties(
        title='Only owners can write and read',
        transitions=('show_members','publish',))
    sdef.setPermission(p_view, 0, (
        r_manager, r_owner))
    sdef.setPermission(p_access, 0, (
        r_manager, r_owner))
    sdef.setPermission(p_modify, 0, (
        r_manager, r_owner))
    sdef.setPermission(p_list, 0, (
        r_manager, r_owner))        

    sdef = wf.states['member']
    sdef.setProperties(
        title='Members (authenticated) can view',
        transitions=('hide', 'publish'))
    sdef.setPermission(p_view, 0, (
        r_auth, r_manager,))
    sdef.setPermission(p_access, 0, (
        r_auth, r_manager,))
    sdef.setPermission(p_modify, 0, (
        r_owner, r_manager,))
    sdef.setPermission(p_list, 0, (
         r_auth,))    

    sdef = wf.states['published']
    sdef.setProperties(
        title='All can view',
        transitions=('hide', 'show_members'))
    sdef.setPermission(p_view, 1, (
        r_anon, r_manager,))
    sdef.setPermission(p_access, 1, (
        r_anon, r_manager,))
    sdef.setPermission(p_modify, 0, (
        r_owner, r_manager,))
    sdef.setPermission(p_list, 1, (
         r_anon,))    

    
    
    # transitions
    tdef = wf.transitions['show_members']
    tdef.setProperties(
        title='Members',
        new_state_id='member',
        actbox_name='Members',
        actbox_url='%(content_url)s/show_members',
        props={'guard_roles':r_owner,})

    tdef = wf.transitions['hide']
    tdef.setProperties(
        title='Hide',
        new_state_id='private',
        actbox_name='Hide',
        actbox_url='%(content_url)s/hide',
        props={'guard_roles':r_owner })

    tdef = wf.transitions['publish']
    tdef.setProperties(
        title='Publish',
        new_state_id='published',
        actbox_name='Publish',
        actbox_url='%(content_url)s/publish',
        props={'guard_roles':r_owner,},)
    

    wf.variables.setStateVar('review_state')

    vdef = wf.variables['action']
    vdef.setProperties(description='The last transition',
                       default_expr='transition/getId|nothing',
                       for_status=1, update_always=1)

    vdef = wf.variables['actor']
    vdef.setProperties(description='The ID of the user who performed '
                       'the last transition',
                       default_expr='user/getId',
                       for_status=1, update_always=1)

    vdef = wf.variables['comments']
    vdef.setProperties(description='Comments about the last transition',
                       default_expr="python:state_change.kwargs.get('comment', '')",
                       for_status=1, update_always=1)

    vdef = wf.variables['review_history']
    vdef.setProperties(description='Provides access to workflow history',
                       default_expr="state_change/getHistory",
                       props={'guard_permissions': p_modify})

    vdef = wf.variables['time']
    vdef.setProperties(description='Time of the last transition',
                       default_expr="state_change/getDateTime",
                       for_status=1, update_always=1)



def createPVSFolderWorkflow(id):
    ob = DCWorkflowDefinition(id)
    setupPVSFolderWorkflow(ob)
    return ob

addWorkflowFactory(createPVSFolderWorkflow, id='pvs_folder_workflow',
                   title='PVS Folder Workflow')

pvs_folder_workflow = 'pvs_folder_workflow (PVS Folder Workflow)'
pvs_folder_workflow_id = 'pvs_folder_workflow'


