"""
PVSWorkflow.py (inspiration: ForumWorkflow.py de CMFBoard)
"""

from Products.CMFCore.CMFCorePermissions import \
     View, ModifyPortalContent, AccessContentsInformation, \
     ReviewPortalContent, RequestReview

from Products.CMFCore.WorkflowTool import addWorkflowFactory
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
from Products.DCWorkflow.Transitions import TRIGGER_AUTOMATIC
from Products.PythonScripts.PythonScript import manage_addPythonScript

import os

from PVSPermissions import CreateVersions

from PVSPermissions import *

p_access = AccessContentsInformation
p_modify = ModifyPortalContent
p_view = View
p_review = ReviewPortalContent
p_request = RequestReview
p_manage = ManageProperties
p_version = CreateVersions

r_anon = 'Anonymous'
r_manager = 'Manager'
r_reviewer = 'Reviewer'
r_owner = 'Owner'



def setupPVSWorkflow(wf):
    wf.setProperties(title='PVS Workflow')

    for s in ('private','pending','published'):
        wf.states.addState(s)
    for t in ('publish','hide','retract','submit'):
        wf.transitions.addTransition(t)
    for v in ('action', 'actor', 'comments', 'review_history', 'time'):
        wf.variables.addVariable(v)
    for p in (p_access, p_view, p_review, p_request, p_modify, p_manage, p_version):
        wf.addManagedPermission(p)

    # states
    wf.states.setInitialState('private')

    sdef = wf.states['private']
    sdef.setProperties(
        title='Only owners can write and read',
        transitions=('submit',))
    sdef.setPermission(p_view, 0, (
        r_manager, r_reviewer, r_owner))
    sdef.setPermission(p_access, 0, (
        r_manager, r_reviewer, r_owner))
    sdef.setPermission(p_modify, 0, (
        r_manager, r_owner))
    sdef.setPermission(p_manage, 0, (
        r_manager, r_owner))

    sdef = wf.states['pending']
    sdef.setProperties(
        title='Owners and Reviewers can retract, Reviewers can publish',
        transitions=('retract', 'publish'))
    sdef.setPermission(p_view, 0, (
        r_owner, r_manager, r_reviewer,))
    sdef.setPermission(p_access, 0, (
        r_owner, r_manager, r_reviewer,))
    sdef.setPermission(p_review, 0, (
        r_manager, r_reviewer))
    sdef.setPermission(p_modify, 0, (
        ))
    sdef.setPermission(p_manage, 0, (
        ))
    sdef.setPermission(p_version, 0, (
        ))



    sdef = wf.states['published']
    sdef.setProperties(
        title='All can view, Reviewers can retract',
        transitions=('hide',))
    sdef.setPermission(p_view, 0, (
        r_anon, r_owner, r_manager, r_reviewer,))
    sdef.setPermission(p_modify, 0, (
        r_manager, r_reviewer))
    sdef.setPermission(p_manage, 0, (
        r_manager, r_reviewer))
    sdef.setPermission(p_version, 0, (
        r_owner,))



    # transitions
    tdef = wf.transitions['submit']
    tdef.setProperties(
        title='Submit',
        new_state_id='pending',
        actbox_name='Submit',
        actbox_url='%(content_url)s/submit',
        props={'guard_roles':r_owner,})

    tdef = wf.transitions['retract']
    tdef.setProperties(
        title='Retract',
        new_state_id='private',
        actbox_name='Retract',
        actbox_url='%(content_url)s/retract',
        props={'guard_roles':r_owner + ';' + r_reviewer})

    tdef = wf.transitions['publish']
    tdef.setProperties(
        title='Publish',
        new_state_id='published',
        actbox_name='Publish',
        actbox_url='%(content_url)s/publish',
        props={'guard_roles':r_reviewer,},)

    tdef = wf.transitions['hide']
    tdef.setProperties(
        title='Hide',
        new_state_id='private',
        actbox_name='Hide',
        actbox_url='%(content_url)s/hide',
        props={'guard_roles':r_reviewer})



    # variables
    wf.variables.setStateVar('review_state')

    vdef = wf.variables['action']
    vdef.setProperties(description='The last transition',
                       default_expr='transition/getId|nothing',
                       for_status=1, update_always=1)

    vdef = wf.variables['actor']
    vdef.setProperties(description='The ID of the user who performed '
                       'the last transition',
                       default_expr='user/getId',
                       for_status=1, update_always=1)

    vdef = wf.variables['comments']
    vdef.setProperties(description='Comments about the last transition',
                       default_expr="python:state_change.kwargs.get('comment', '')",
                       for_status=1, update_always=1)

    vdef = wf.variables['review_history']
    vdef.setProperties(description='Provides access to workflow history',
                       default_expr="state_change/getHistory",
                       props={'guard_permissions': p_manage})

    vdef = wf.variables['time']
    vdef.setProperties(description='Time of the last transition',
                       default_expr="state_change/getDateTime",
                       for_status=1, update_always=1)


    # worklists
    wf.worklists.addWorklist('reviewer_queue')
    ldef = wf.worklists['reviewer_queue']
    ldef.setProperties('Reviewer tasks', actbox_name='Pending (%(count)d)', actbox_url='%(portal_url)s/search?review_state=pending', actbox_category='global',props={'var_match_review_state':'pending','guard_permissions':'Review portal content',})



def createPVSWorkflow(id):
    ob = DCWorkflowDefinition(id)
    setupPVSWorkflow(ob)
    return ob

addWorkflowFactory(createPVSWorkflow, id='pvs_workflow',
                   title='PVS Workflow')

pvs_workflow = 'pvs_workflow (PVS Workflow)'
pvs_workflow_id = 'pvs_workflow'


