from Acquisition import aq_base, aq_inner, aq_parent
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Acquisition import aq_chain

def patch(klass, method):
    mid = method.__name__
    ori_mid = '_original_' + mid
    if not hasattr(klass, ori_mid):
        setattr(klass, ori_mid, getattr(klass, mid))
    setattr(klass, mid, method)

#---------------------------- patches for CMFCatalogAware -------------------------------------
#

from Products.CMFCore.CMFCatalogAware import CMFCatalogAware

def manage_afterAdd(self, item, container):
    """ Perform version related operations
    """
    if hasattr(self, '_v_version_bundle_'):
        front_end_id = self.getId()
        for version in self._v_version_bundle_:
            # rename all versions, in case it's a rename operation
            v_id = '%s~v~%s' % (self.getId(), version.version_number)
            version._setId(v_id)
            container._setOb(v_id, version)
            version = container._getOb(v_id)
            version.manage_afterAdd(version, container)
        delattr(self, '_v_version_bundle_')
    self._original_manage_afterAdd(item, container)

def manage_afterClone(self, item):
    """ Remove version infomation
    """
    if self.getId().find('~v~') < 0 and self.version_cluster_id > 0:
        self.version_cluster_id = -1
    self._original_manage_afterClone(item)

def manage_beforeDelete(self, item, container):
    """ Bundle versions under this object
    """
    # due to a bug in CMFCatalogAware.__recurse,
    # the container passed here might not be the real one
    container = aq_parent(aq_inner(self))
    pvs = getToolByName(self, 'portal_content_versioning_system', None)
    if self.getId().find('~v~') < 0 and pvs is not None:
        self._v_version_bundle_ = [] # declared as volatile by intension
        for id in pvs.getAllVersionIds(self):
            # find out all hidden versions
            if id.find('~v~') > 0 :
                self._v_version_bundle_.append(aq_base(container._getOb(id)))
                container._delObject(id)
    self._original_manage_beforeDelete(item, container)

CMFCatalogAware.security = ClassSecurityInfo()
CMFCatalogAware.security.declareProtected('Modify portal content', 'manage_suicide')
from zExceptions import Unauthorized
def manage_suicide(self):
    """ For obj to delete itself
    """
    if hasattr(self, 'getDeleteObjectInfo'):
        if not self.getDeleteObjectInfo(self, 1):
            raise Unauthorized
    container = aq_parent(aq_inner(self))
    return container.manage_delObjects(ids=[self.getId()])

patch(CMFCatalogAware, manage_afterAdd)
patch(CMFCatalogAware, manage_afterClone)
patch(CMFCatalogAware, manage_beforeDelete)
CMFCatalogAware.manage_suicide = manage_suicide

try:
    from Products.Archetypes.BaseObject import BaseObject
    patch(BaseObject, manage_afterAdd)
    patch(BaseObject, manage_afterClone)
    patch(BaseObject, manage_beforeDelete)
    BaseObject.manage_suicide = manage_suicide
    BaseObject.version_cluster_id = -1
    InitializeClass(BaseObject)
except:
    pass

# default version_cluster_id for all portal objects
CMFCatalogAware.version_cluster_id = -1
InitializeClass(CMFCatalogAware)

#---------------------------- patches for contentEdit-------------------------------------
#

from Products.CMFPlone.PloneTool import PloneTool

def contentEdit(self, obj, **kwargs):
    # don't rename obj if it's an hidden version
    container = aq_parent(aq_inner(obj))
    if hasattr(container, 'objectIds') and obj.getId() not in container.objectIds():
        kwargs['id'] = None
    return self._original_contentEdit(obj, **kwargs)

patch(PloneTool, contentEdit)
#InitializeClass(PloneTool)

def _unauthorized(self):
    req = self.REQUEST
    resp = self._cleanupResponse()
    if hasattr(self, 'portal_content_versioning_system') and req.URL.endswith('edit_form') and len(req.form) > 0:
        context = req.PARENTS[0]
        edit_method_name = req.PUBLISHED.getId()
        edit_method_name = edit_method_name[:-5] # get rid of the '_form'
        edit_method = getattr(self, edit_method_name)
        new_context = self.portal_content_versioning_system.createVersion(context)
        edit_method = edit_method.__of__(new_context)
        mapply(edit_method, req.args, req,
                call_object,1,
                missing_name,
                dont_publish_class,
                req, bind=1)
        get_transaction().commit()
        req.RESPONSE.redirect(new_context.absolute_url() + '?portal_status_message=New+version+saved.', lock=1)
        return
    else:
        return self._original__unauthorized()


#patch(CookieCrumbler, _unauthorized)
