
import string
import zLOG

from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
from Products.CMFCore.utils import getToolByName
from config import PROJECTNAME

def send_email_review(context, state_info,):
    """Send email to Reviewer (when submit)."""

    plone_utils = getToolByName(context, 'plone_utils')
    portal_membership = getToolByName(context, 'portal_membership')
    acl_users = getToolByName(context, 'acl_users')
    
    # default roles from acl_users
    users_ids = list(acl_users.getUsersOfRole('Reviewer',object=state_info.object))
    emails_to = []

    # roles on the object
    iroles = plone_utils.getInheritedLocalRoles(state_info.object)
    for irole in iroles:
        for role in  irole[1]:
            if role == 'Reviewer':
                if irole[2] == 'group':
                    users_ids = users_ids + acl_users.getUserMemberIds(irole[0])
                if irole[2] == 'user':
                    users_ids.append(irole[0])

    # emails collect 
    for user_id in users_ids:
        user = portal_membership.getMemberById(user_id)
        if user.getEmail() not in emails_to:
            emails_to.append(user.getEmail())
        

    user = portal_membership.getAuthenticatedMember()
    if user:
        user_name = user.getUserName()
    else:
        user_name = "(User unknown)"

    url = state_info.object.absolute_url()
    title = state_info.object.Title()


    emails = string.join(emails_to,', ')

    portal = getToolByName(context, 'portal_url').getPortalObject()
    
    email_from_address = portal.getProperty('email_from_address')
    email_from_name = portal.getProperty('email_from_name')
    email_from = email_from_name + ' <' + email_from_address + '> '
    portal_title = portal.getProperty('title')

    pmt = portal.portal_mailtemplates
    mspool = portal.portal_mailspool
    info = {
            'subject': "Plone Pvs Review %s" % (title,),
            'user': user_name,
            'title': title,
            'url': url,
            }
    headers={
        'To': emails,
        }
    files=[]
    
    message = pmt.createMessage(
        PROJECTNAME, 'pvs_message_review', info, headers, files=files)
    mspool.sendmail(email_from, emails, message)


    



def send_email_published(context, state_info,):
    """Send email to actor mading latest transaction : content is published"""

    portal_workflow = getToolByName(context, 'portal_workflow')
    portal_membership = getToolByName(context, 'portal_membership')

    actor_submit_id = portal_workflow.getInfoFor(state_info.object, 'actor')
    actor_submit = portal_membership.getMemberById(actor_submit_id)

    # actor can be null : a zope's user
    if actor_submit != None:
        actor_email = actor_submit.getEmail()

        url = state_info.object.absolute_url()
        title = state_info.object.Title()

        portal = getToolByName(context, 'portal_url').getPortalObject()
        
        email_from_address = portal.getProperty('email_from_address')
        email_from_name = portal.getProperty('email_from_name')
        email_from = email_from_name + ' <' + email_from_address + '> '
        portal_title = portal.getProperty('title')

        pmt = portal.portal_mailtemplates
        mspool = portal.portal_mailspool
        info = {
            'subject': "Plone Pvs Publish %s" % (title,),
            'title': title,
            'url': url,
            }
        headers={
            'To': actor_email,
            }
        files=[]
        
        message = pmt.createMessage(
            PROJECTNAME, 'pvs_message_publish', info, headers, files=files)
        mspool.sendmail(email_from, actor_email, message)



def send_email_retract(context, state_info,):
    """Send email to actor mading latest transaction : content is retracted"""

    portal_workflow = getToolByName(context, 'portal_workflow')
    portal_membership = getToolByName(context, 'portal_membership')

    actor_submit_id = portal_workflow.getInfoFor(state_info.object, 'actor')
    actor_submit = portal_membership.getMemberById(actor_submit_id)

    # actor can be null : a zope's user
    if actor_submit != None:    
        actor_email = actor_submit.getEmail()

        url = state_info.object.absolute_url()
        title = state_info.object.Title()
        
        portal = getToolByName(context, 'portal_url').getPortalObject()

        email_from_address = portal.getProperty('email_from_address')
        email_from_name = portal.getProperty('email_from_name')
        email_from = email_from_name + ' <' + email_from_address + '> '
        portal_title = portal.getProperty('title')


        pmt = portal.portal_mailtemplates
        mspool = portal.portal_mailspool
        info = {
            'subject': "Plone Pvs Retract %s" % (title,),
            'title': title,
            'url': url,
            }
        headers={
            'To': actor_email,
            }
        files=[]
        
        message = pmt.createMessage(
            PROJECTNAME, 'pvs_message_retract', info, headers, files=files)
        mspool.sendmail(email_from, actor_email, message)
        

